-- http://lua-users.org/wiki/LuaXml
-----------------------------------------------------------------------------------------
-- LUA only XmlParser from Alexander Makeev
-----------------------------------------------------------------------------------------

local	XmlParser = {};

function XmlParser:toXmlString(value)
	value = string.gsub (value, "&", "&amp;");		-- '&' -> "&amp;"
	value = string.gsub (value, "<", "&lt;");		-- '<' -> "&lt;"
	value = string.gsub (value, ">", "&gt;");		-- '>' -> "&gt;"
	--value = string.gsub (value, "'", "&apos;");	-- '\'' -> "&apos;"
	value = string.gsub (value, "\"", "&quot;");	-- '"' -> "&quot;"
	-- replace non printable char -> "&#xD;"
   	value = string.gsub(value, "([^%w%&%;%p%\t% ])",
       	function (c) 
       		return string.format("&#x%X;", string.byte(c)) 
       		--return string.format("&#x%02X;", string.byte(c)) 
       		--return string.format("&#%02d;", string.byte(c)) 
       	end);
	return value;
end

function XmlParser:fromXmlString(value)
  	value = string.gsub(value, "&#x([%x]+)%;",
      	function(h) 
      		return string.char(tonumber(h,16)) 
      	end);
  	value = string.gsub(value, "&#([0-9]+)%;",
      	function(h) 
      		return string.char(tonumber(h,10)) 
      	end);
	value = string.gsub (value, "&quot;", "\"");
	value = string.gsub (value, "&apos;", "'");
	value = string.gsub (value, "&gt;", ">");
	value = string.gsub (value, "&lt;", "<");
	value = string.gsub (value, "&amp;", "&");
	return value;
end
   
function XmlParser:parseArgs(s)
  local arg = {}
  string.gsub(s, "(%w+)=([\"'])(.-)%2", function (w, _, a)
    	arg[w] = self:fromXmlString(a);
  	end)
  return arg
end

function XmlParser:parseXmlText(xmlText)
  local stack = {}
  local top = {name=nil,value=nil,attributes={},childNodes={}}
  table.insert(stack, top)
  local ni,c,label,xarg, empty
  local i, j = 1, 1
  while true do
    ni,j,c,label,xarg, empty = string.find(xmlText, "<(%/?)([%w:]+)(.-)(%/?)>", i)
    if not ni then break end
    local text = string.sub(xmlText, i, ni-1);
    if not string.find(text, "^%s*$") then
      top.value=(top.value or "")..self:fromXmlString(text);
    end
    if empty == "/" then  -- empty element tag
      table.insert(top.childNodes, {name=label,value=nil,attributes=self:parseArgs(xarg),childNodes={}})
    elseif c == "" then   -- start tag
      top = {name=label, value=nil, attributes=self:parseArgs(xarg), childNodes={}}
      table.insert(stack, top)   -- new level
      --log("openTag ="..top.name);
    else  -- end tag
      local toclose = table.remove(stack)  -- remove top
      --log("closeTag="..toclose.name);
      top = stack[#stack]
      if #stack < 1 then
        error("XmlParser: nothing to close with "..label)
      end
      if toclose.name ~= label then
        error("XmlParser: trying to close "..toclose.name.." with "..label)
      end
      table.insert(top.childNodes, toclose)
    end
    i = j+1
  end
  local text = string.sub(xmlText, i);
  if not string.find(text, "^%s*$") then
      stack[#stack].value=(stack[#stack].value or "")..self:fromXmlString(text);
  end
  if #stack > 1 then
    error("XmlParser: unclosed "..stack[stack.n].name)
  end
  return stack[1].childNodes[1];
end

function XmlParser:parseXmlFile(xmlFileName)
	if love then
		local	xmlText = love.filesystem.read(xmlFileName)
        return self:parseXmlText(xmlText),nil;
	else
		
		local hFile,err = io.open(xmlFileName,"r");
		if (not err) then
			local xmlText=hFile:read("*a"); -- read file content
			io.close(hFile);
	        return self:parseXmlText(xmlText),nil;
		else
			return nil,err;
		end
	end
end

function XmlParser:printNode(node, step)
	step = step or 0
	local	repstr = string.rep("  ", step)
	print(string.format("%sname> %s", repstr, node.name))
	for key, attribute in pairs(node.attributes) do
		print(string.format("%sattr> %s = %s", repstr, key, attribute))
	end
	if node.value then
		print(string.format("%sdata> %s", repstr, node.value))
	end
	for key, child in pairs(node.childNodes) do
		XmlParser:printNode(child, step + 1)
	end
end

return XmlParser
